{% extends 'debug_toolbar/base.html' %}
{% load static %}

{% block css %}{{ block.super }}
  <link rel="stylesheet" href="{% static 'django_logikal/fonts.css' %}">
  <style nonce="debug-toolbar">
    :root {
      --djdt-font-family-primary: 'lato', sans-serif;
      --djdt-font-family-monospace: 'roboto-mono', monospace;
    }
  </style>
{% endblock %}
