{% extends 'admin/base.html' %}
{% load static %}

{% block extrastyle %}{{ block.super }}
  <link rel="stylesheet" href="{% static 'django_logikal/fonts.css' %}">
  <style nonce="{{ request.csp_nonce }}">
    html[data-theme='light'],
    :root {
      --font-family-primary: 'lato', sans-serif;
      --font-family-monospace: 'roboto-mono', monospace;
      font-size: 18px;
    }
  </style>
{% endblock %}
