/* Sans-serif font | License: url('fonts/Lato-LICENSE.txt') */
@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-Thin.woff2') format('woff2'),
    url('fonts/Lato-Thin.woff') format('woff'),
    url('fonts/Lato-Thin.ttf') format('truetype');
  font-weight: 200;
  font-style: normal;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-ThinItalic.woff2') format('woff2'),
    url('fonts/Lato-ThinItalic.woff') format('woff'),
    url('fonts/Lato-ThinItalic.ttf') format('truetype');
  font-weight: 200;
  font-style: italic;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-Light.woff2') format('woff2'),
    url('fonts/Lato-Light.woff') format('woff'),
    url('fonts/Lato-Light.ttf') format('truetype');
  font-weight: 300;
  font-style: normal;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-LightItalic.woff2') format('woff2'),
    url('fonts/Lato-LightItalic.woff') format('woff'),
    url('fonts/Lato-LightItalic.ttf') format('truetype');
  font-weight: 300;
  font-style: italic;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-Regular.woff2') format('woff2'),
    url('fonts/Lato-Regular.woff') format('woff'),
    url('fonts/Lato-Regular.ttf') format('truetype');
  font-weight: normal;
  font-style: normal;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-Italic.woff2') format('woff2'),
    url('fonts/Lato-Italic.woff') format('woff'),
    url('fonts/Lato-Italic.ttf') format('truetype');
  font-weight: normal;
  font-style: italic;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-Semibold.woff2') format('woff2'),
    url('fonts/Lato-Semibold.woff') format('woff'),
    url('fonts/Lato-Semibold.ttf') format('truetype');
  font-weight: 600;
  font-style: normal;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-SemiboldItalic.woff2') format('woff2'),
    url('fonts/Lato-SemiboldItalic.woff') format('woff'),
    url('fonts/Lato-SemiboldItalic.ttf') format('truetype');
  font-weight: 600;
  font-style: italic;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-Bold.woff2') format('woff2'),
    url('fonts/Lato-Bold.woff') format('woff'),
    url('fonts/Lato-Bold.ttf') format('truetype');
  font-weight: bold;
  font-style: normal;
}

@font-face {
  font-family: 'lato';
  src:
    url('fonts/Lato-BoldItalic.woff2') format('woff2'),
    url('fonts/Lato-BoldItalic.woff') format('woff'),
    url('fonts/Lato-BoldItalic.ttf') format('truetype');
  font-weight: bold;
  font-style: italic;
}

/* Monospace font | License: url('fonts/RobotoMono-LICENSE.txt') */
@font-face {
  font-family: 'roboto-mono';
  src:
    url('fonts/RobotoMono-Regular.woff2') format('woff2'),
    url('fonts/RobotoMono-Regular.woff') format('woff'),
    url('fonts/RobotoMono-Regular.ttf') format('truetype');
  font-weight: normal;
  font-style: normal;
}

@font-face {
  font-family: 'roboto-mono';
  src:
    url('fonts/RobotoMono-Italic.woff2') format('woff2'),
    url('fonts/RobotoMono-Italic.woff') format('woff'),
    url('fonts/RobotoMono-Italic.ttf') format('truetype');
  font-weight: normal;
  font-style: italic;
}

@font-face {
  font-family: 'roboto-mono';
  src:
    url('fonts/RobotoMono-Bold.woff2') format('woff2'),
    url('fonts/RobotoMono-Bold.woff') format('woff'),
    url('fonts/RobotoMono-Bold.ttf') format('truetype');
  font-weight: bold;
  font-style: normal;
}

@font-face {
  font-family: 'roboto-mono';
  src:
    url('fonts/RobotoMono-BoldItalic.woff2') format('woff2'),
    url('fonts/RobotoMono-BoldItalic.woff') format('woff'),
    url('fonts/RobotoMono-BoldItalic.ttf') format('truetype');
  font-weight: bold;
  font-style: italic;
}
